@@ -19,6 +19,7 @@
 
 #include <string.h>
 #include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
 #include "gailentry.h"
 #include "gailcombo.h"
 #include <libgail-util/gailmisc.h>
@@ -147,6 +148,8 @@ static gboolean              idle_do_action              (gpointer        data);
 static gint                  gail_entry_get_n_actions    (AtkAction       *action);
 static G_CONST_RETURN gchar* gail_entry_get_description  (AtkAction       *action,
                                                           gint            i);
+static G_CONST_RETURN gchar* gail_entry_get_keybinding   (AtkAction       *action,
+                                                          gint            i);
 static G_CONST_RETURN gchar* gail_entry_action_get_name  (AtkAction       *action,
                                                           gint            i);
 static gboolean              gail_entry_set_description  (AtkAction       *action,
@@ -245,6 +248,7 @@ gail_entry_object_init (GailEntry *entry)
   entry->cursor_position = 0;
   entry->selection_bound = 0;
   entry->activate_description = NULL;
+  entry->activate_keybinding = NULL;
 }
 
 static void
@@ -379,6 +383,7 @@ gail_entry_finalize (GObject            *object)
 
   g_object_unref (entry->textutil);
   g_free (entry->activate_description);
+  g_free (entry->activate_keybinding);
   if (entry->action_idle_handler)
     {
       g_source_remove (entry->action_idle_handler);
@@ -1102,7 +1107,7 @@ _gail_entry_insert_text_cb (GtkEntry *entry,
    * The signal will be emitted when the cursor position is updated.
    * or in an idle handler if it not updated.
    */
-   if (gail_entry->insert_idle_handler)
+   if (gail_entry->insert_idle_handler == 0)
      gail_entry->insert_idle_handler = g_idle_add (idle_notify_insert, gail_entry);
 }
 
@@ -1210,6 +1215,7 @@ atk_action_interface_init (AtkActionIface *iface)
   iface->do_action = gail_entry_do_action;
   iface->get_n_actions = gail_entry_get_n_actions;
   iface->get_description = gail_entry_get_description;
+  iface->get_keybinding = gail_entry_get_keybinding;
   iface->get_name = gail_entry_action_get_name;
   iface->set_description = gail_entry_set_description;
 }
@@ -1296,6 +1302,72 @@ gail_entry_get_description (AtkAction *action,
   return return_value; 
 }
 
+static G_CONST_RETURN gchar*
+gail_entry_get_keybinding (AtkAction *action,
+                           gint      i)
+{
+  GailEntry *entry;
+  gchar *return_value = NULL;
+
+  entry = GAIL_ENTRY (action);
+  switch (i)
+    {
+    case 0:
+      {
+        /*
+         * We look for a mnemonic on the label
+         */
+        GtkWidget *widget;
+        GtkWidget *label;
+        AtkRelationSet *set;
+        AtkRelation *relation;
+        GPtrArray *target;
+        gpointer target_object;
+        guint key_val; 
+
+        entry = GAIL_ENTRY (action);
+        widget = GTK_ACCESSIBLE (entry)->widget;
+        if (widget == NULL)
+          /*
+           * State is defunct
+           */
+          return NULL;
+
+        /* Find labelled-by relation */
+
+        set = atk_object_ref_relation_set (ATK_OBJECT (action));
+        if (!set)
+          return NULL;
+        relation = atk_relation_set_get_relation_by_type (set, ATK_RELATION_LABELLED_BY);
+        if (relation)
+          {              
+            target = atk_relation_get_target (relation);
+          
+            target_object = g_ptr_array_index (target, 0);
+            if (GTK_IS_ACCESSIBLE (target_object))
+              {
+                label = GTK_ACCESSIBLE (target_object)->widget;
+              } 
+          }
+        g_object_unref (set);
+
+        if (GTK_IS_LABEL (label))
+          {
+            key_val = gtk_label_get_mnemonic_keyval (GTK_LABEL (label)); 
+            if (key_val != GDK_VoidSymbol)
+              return_value = gtk_accelerator_name (key_val, GDK_MOD1_MASK);
+          }
+        g_free (entry->activate_keybinding);
+        entry->activate_keybinding = return_value;
+        break;
+      }
+    default:
+      return_value = NULL;
+      break;
+    }
+  return return_value; 
+}
+
 static G_CONST_RETURN gchar*
 gail_entry_action_get_name (AtkAction *action,
                             gint      i)
